@@ -30,8 +30,11 @@
 
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/navigation.h>
+#include <gst/pbutils/missing-plugins.h>
+#include <gst/pbutils/install-plugins.h>
 
 #include <gst/video/video.h>
+#include <gst/tag/tag.h>
 
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
@@ -41,7 +44,6 @@
 #include "parole-gst.h"
 
 #include "common/parole-common.h"
-#include "common/parole-rc-utils.h"
 
 #include "parole-utils.h"
 
@@ -61,8 +63,9 @@ static void     parole_gst_change_state 	(ParoleGst *gst,
 static void	parole_gst_terminate_internal   (ParoleGst *gst, 
 						 gboolean fade_sound);
 						 
-static void     parole_gst_seek_cdda_track	(ParoleGst *gst,
-						 gint track);
+static gchar * parole_gst_tag_list_get_cover_external (ParoleGst *gst);
+						 
+static GdkPixbuf * parole_gst_tag_list_get_cover (ParoleGst *gst, GstTagList *tag_list);
 
 typedef enum 
 {
@@ -79,6 +82,15 @@ typedef enum
 
 } GstPlayFlags;
 
+enum
+{
+    GST_DVD_ROOT_MENU,
+    GST_DVD_TITLE_MENU,
+    GST_DVD_AUDIO_MENU,
+    GST_DVD_ANGLE_MENU,
+    GST_DVD_CHAPTER_MENU
+};
+
 struct ParoleGstPrivate
 {
     GstElement	 *playbin;
@@ -108,13 +120,16 @@ struct ParoleGstPrivate
     gboolean      buffering;
     gboolean      update_color_balance;
     
+    gboolean      use_custom_subtitles;
+    gchar*        custom_subtitles;
+    
     ParoleAspectRatio aspect_ratio;
     gulong	  state_change_id;
     
     gboolean	  scale_logo;
     
     /*
-     * xvimage sink has brightness+hue+aturation+contrast.
+     * xvimage sink has brightness+hue+saturation+contrast.
      */
     gboolean	  xvimage_sink;
     
@@ -129,6 +144,8 @@ enum
     MEDIA_TAG,
     BUFFERING,
     ERROR,
+    DVD_CHAPTER_CHANGE,
+    DVD_CHAPTER_COUNT_CHANGE,
     LAST_SIGNAL
 };
 
@@ -554,6 +571,7 @@ parole_gst_load_logo (ParoleGst *gst)
     gchar *path;
 
     path = g_strdup_printf ("%s/parole.png", PIXMAPS_DIR);
+    
     gst->priv->logo = gdk_pixbuf_new_from_file (path, NULL);
     g_free (path);
 }
@@ -994,6 +1012,217 @@ out:
     ;
 }
 
+static GdkPixbuf *
+parole_gst_buffer_to_pixbuf (GstBuffer *buffer)
+{
+  GdkPixbufLoader *loader;
+  GdkPixbuf *pixbuf = NULL;
+  GError *err = NULL;
+
+  loader = gdk_pixbuf_loader_new ();
+
+  if (gdk_pixbuf_loader_write (loader, buffer->data, buffer->size, &err) &&
+      gdk_pixbuf_loader_close (loader, &err)) {
+    pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+    if (pixbuf)
+      g_object_ref (pixbuf);
+  } else {
+    GST_WARNING("could not convert tag image to pixbuf: %s", err->message);
+    g_error_free (err);
+  }
+
+  g_object_unref (loader);
+  return pixbuf;
+}
+
+gchar *
+parole_gst_tag_list_get_cover_external (ParoleGst *gst)
+{
+    gchar *uri;
+    gchar *filename;
+    gchar *directory;
+    GDir  *file_dir;
+    GError *error = NULL;
+    const gchar *listing = NULL;
+    gchar *lower = NULL;
+    gchar *cover = NULL;
+    gchar *cover_filename = NULL;
+    
+    g_object_get (G_OBJECT (gst->priv->stream),
+		  "uri", &uri,
+		  NULL);
+		  
+    filename = g_filename_from_uri(uri, NULL, NULL);
+    
+    directory = g_path_get_dirname(filename);
+    
+    file_dir = g_dir_open(directory, 0, &error);
+    if (error)
+    {
+        g_error_free (error);
+        return NULL;
+    }
+    
+    while ( (listing = g_dir_read_name(file_dir)) )
+    {
+        lower = g_utf8_strdown(listing, -1);
+        if ( g_strcmp0(lower, "cover.jpg") == 0 )
+            cover = g_strdup(listing);
+        else if ( g_strcmp0(lower, "folder.jpg") == 0 )
+            cover = g_strdup(listing);
+        else if ( g_strcmp0(lower, "album.jpg") == 0 )
+            cover = g_strdup(listing);
+        else if ( g_strcmp0(lower, "thumb.jpg") == 0 )
+            cover = g_strdup(listing);
+        else if ( g_strcmp0(lower, "albumartsmall.jpg") == 0 )
+            cover = g_strdup(listing);
+            
+        if (cover)
+        {
+            cover_filename = g_build_filename(directory, cover, NULL);
+            break;
+        }
+    }
+    g_free(uri);
+    g_free(filename);
+    g_free(directory);
+    g_free(file_dir);
+    g_free(lower);
+    return cover_filename;
+}
+
+static const GValue *
+parole_gst_tag_list_get_cover_real (GstTagList *tag_list)
+{
+  const GValue *cover_value = NULL;
+  guint i;
+
+  for (i = 0; ; i++) {
+    const GValue *value;
+    GstBuffer *buffer;
+    GstStructure *caps_struct;
+    int type;
+
+    value = gst_tag_list_get_value_index (tag_list,
+					  GST_TAG_IMAGE,
+					  i);
+    if (value == NULL)
+      break;
+
+    buffer = gst_value_get_buffer (value);
+
+    caps_struct = gst_caps_get_structure (buffer->caps, 0);
+    
+    gst_structure_get_enum (caps_struct,
+			    "image-type",
+			    GST_TYPE_TAG_IMAGE_TYPE,
+			    &type);
+    if (type == GST_TAG_IMAGE_TYPE_UNDEFINED) {
+      if (cover_value == NULL)
+        cover_value = value;
+    } else if (type == GST_TAG_IMAGE_TYPE_FRONT_COVER) {
+      cover_value = value;
+      break;
+    }
+  }
+
+  return cover_value;
+}
+
+GdkPixbuf *
+parole_gst_tag_list_get_cover (ParoleGst *gst, GstTagList *tag_list)
+{
+  gchar *cover_filename;
+  const GValue *cover_value;
+  
+
+  g_return_val_if_fail (tag_list != NULL, FALSE);
+  
+  cover_filename = parole_gst_tag_list_get_cover_external(gst);
+  if (cover_filename)
+  {
+    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size(cover_filename, 256, 256, NULL);
+    if (pixbuf)
+        return pixbuf;
+  }
+
+  cover_value = parole_gst_tag_list_get_cover_real (tag_list);
+  /* Fallback to preview */
+  if (!cover_value) {
+    cover_value = gst_tag_list_get_value_index (tag_list,
+						GST_TAG_PREVIEW_IMAGE,
+						0);
+  }
+
+  if (cover_value) {
+    GstBuffer *buffer;
+    GdkPixbuf *pixbuf;
+
+    buffer = gst_value_get_buffer (cover_value);
+    pixbuf = parole_gst_buffer_to_pixbuf (buffer);
+    return pixbuf;
+  }
+
+  return NULL;
+}
+
+static void
+parole_gst_get_meta_data_dvd (ParoleGst *gst)
+{
+    gint n_chapters;
+    guint num_chapters = 1;
+    guint chapter = 1;
+    guint current_num_chapters;
+    guint current_chapter;
+    gint64 val = -1;
+    
+    GstFormat format;
+    
+    g_object_get (  G_OBJECT (gst->priv->stream),
+                    "num-tracks", &current_num_chapters,
+		            "track", &current_chapter,
+		            NULL);
+
+    format = gst_format_get_by_nick ("chapter");
+
+    /* Get the number of chapters for the current title. */
+    if ( gst_element_query_duration (gst->priv->playbin, &format, &val) )
+    {
+        n_chapters = (gint) val;
+        num_chapters = (guint) n_chapters;
+        if (num_chapters != current_num_chapters)
+        {
+            g_object_set (G_OBJECT (gst->priv->stream),
+		              "num-tracks", num_chapters,
+		              NULL);
+            TRACE("Updated DVD chapter count: %i", n_chapters);
+            
+            g_signal_emit (G_OBJECT (gst), signals [DVD_CHAPTER_COUNT_CHANGE], 0, 
+                n_chapters);
+        }
+    }
+    
+    /* Get the current chapter. */
+    val = -1;
+    if ( gst_element_query_position (gst->priv->playbin, &format, &val) )
+    {
+        chapter = (guint)(gint) val;
+        if ( chapter != current_chapter || num_chapters != 1 )
+        {
+            g_object_set (G_OBJECT (gst->priv->stream),
+		              "track", chapter,
+		              NULL);
+            TRACE("Updated current DVD chapter: %i", chapter);
+            
+            if (current_chapter != 1)
+            g_signal_emit (G_OBJECT (gst), signals [DVD_CHAPTER_CHANGE], 0, 
+                chapter);
+        }
+    }
+    
+}
+
+
 static void
 parole_gst_get_meta_data_cdda (ParoleGst *gst, GstTagList *tag)
 {
@@ -1006,7 +1235,18 @@ parole_gst_get_meta_data_cdda (ParoleGst *gst, GstTagList *tag)
 	g_object_set (G_OBJECT (gst->priv->stream),
 		      "num-tracks", num_tracks,
 		      "track", track,
+		      "title", g_strdup_printf(_("Track %i"), track),
+		      "artist", NULL,
+		      "year", NULL,
+		      "album", _("Audio CD"),
+		      "comment", NULL,
 		      NULL);
+		      
+    parole_stream_set_image (G_OBJECT (gst->priv->stream), NULL);
+    g_object_set (G_OBJECT (gst->priv->stream),
+		  "tag-available", FALSE,
+		  NULL);
+    
 	TRACE ("num_tracks=%i track=%i", num_tracks, track);
 	g_signal_emit (G_OBJECT (gst), signals [MEDIA_TAG], 0, gst->priv->stream);
     }
@@ -1017,6 +1257,7 @@ parole_gst_get_meta_data_local_file (ParoleGst *gst, GstTagList *tag)
 {
     gchar *str;
     GDate *date;
+    GdkPixbuf *pixbuf;
     
     if ( gst_tag_list_get_string_index (tag, GST_TAG_TITLE, 0, &str) )
     {
@@ -1066,6 +1307,13 @@ parole_gst_get_meta_data_local_file (ParoleGst *gst, GstTagList *tag)
 		      NULL);
 	g_free (str);
     }
+    
+    pixbuf = parole_gst_tag_list_get_cover (gst, tag);
+    if (pixbuf)
+    {
+	    parole_stream_set_image (G_OBJECT (gst->priv->stream), pixbuf);
+	    g_object_unref (pixbuf);
+    }
 
     g_object_set (G_OBJECT (gst->priv->stream),
 		  "tag-available", TRUE,
@@ -1075,6 +1323,26 @@ parole_gst_get_meta_data_local_file (ParoleGst *gst, GstTagList *tag)
     
 }
 
+static void
+parole_gst_get_meta_data_unknown (ParoleGst *gst)
+{
+	g_object_set (G_OBJECT (gst->priv->stream),
+		      "title", NULL,
+		      "artist", NULL,
+		      "year", NULL,
+		      "album", NULL,
+		      "comment", NULL,
+		      NULL);
+    
+    parole_stream_set_image (G_OBJECT (gst->priv->stream), NULL);
+
+    g_object_set (G_OBJECT (gst->priv->stream),
+		  "tag-available", FALSE,
+		  NULL);
+		  
+    g_signal_emit (G_OBJECT (gst), signals [MEDIA_TAG], 0, gst->priv->stream);
+}
+
 static void
 parole_gst_get_meta_data (ParoleGst *gst, GstTagList *tag)
 {
@@ -1083,7 +1351,7 @@ parole_gst_get_meta_data (ParoleGst *gst, GstTagList *tag)
     g_object_get (G_OBJECT (gst->priv->stream),
 		  "media-type", &media_type,
 		  NULL);
-    
+		  
     switch ( media_type )
     {
 	case PAROLE_MEDIA_TYPE_LOCAL_FILE:
@@ -1092,6 +1360,13 @@ parole_gst_get_meta_data (ParoleGst *gst, GstTagList *tag)
 	case PAROLE_MEDIA_TYPE_CDDA:
 	    parole_gst_get_meta_data_cdda (gst, tag);
 	    break;
+    case PAROLE_MEDIA_TYPE_DVD:
+        parole_gst_get_meta_data_dvd (gst);
+        break;
+    case PAROLE_MEDIA_TYPE_REMOTE:
+    case PAROLE_MEDIA_TYPE_UNKNOWN:
+	    parole_gst_get_meta_data_unknown (gst);
+	    break;
 	default:
 	    break;
     }
@@ -1119,10 +1394,76 @@ parole_gst_application_message (ParoleGst *gst, GstMessage *msg)
     }
 }
 
+/**
+ * parole_gst_install_plugins_result_func:
+ * @result    : %GST_INSTALL_PLUGINS_SUCCESS (0) if successful.
+ * @user_data : gst instance passed as user_data for callback function.
+ *
+ * Callback function for when asynchronous codec installation finishes.  Update
+ * the gstreamer plugin registry and restart playback.
+ **/
+static void
+parole_gst_install_plugins_result_func(GstInstallPluginsReturn result, gpointer user_data)
+{
+    switch (result)
+    {
+        case GST_INSTALL_PLUGINS_SUCCESS:
+            g_debug ("Finished plugin install.\n");
+            gst_update_registry();
+            parole_gst_resume(PAROLE_GST(user_data));
+            break;
+        case GST_INSTALL_PLUGINS_NOT_FOUND:
+            g_debug ("An appropriate installation candidate could not be found.\n");
+            break;
+        case GST_INSTALL_PLUGINS_USER_ABORT:
+            g_debug ("User aborted plugin install.\n");
+            break;
+        default:
+            g_debug ("Plugin installation failed with code %i\n", result);
+            break;
+    }
+}
+
+static GtkDialog*
+parole_gst_missing_codec_dialog(ParoleGst *gst, GstMessage *msg)
+{
+    GtkMessageDialog *dialog;
+    gchar*     desc;
+                                                    
+    desc = gst_missing_plugin_message_get_description(msg);
+    
+    dialog = GTK_MESSAGE_DIALOG(gtk_message_dialog_new_with_markup(
+                            NULL,
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_MESSAGE_QUESTION,
+                            GTK_BUTTONS_NONE,
+                            "<b><big>%s</big></b>", 
+                            _("Additional software is required.")
+                            ));
+                            
+    gtk_dialog_add_buttons( GTK_DIALOG(dialog), 
+                            _("Don't Install"),
+                            GTK_RESPONSE_REJECT,
+                            _("Install"), 
+                            GTK_RESPONSE_ACCEPT,
+                            NULL );
+    
+    gtk_message_dialog_format_secondary_markup(dialog,
+                                             "Parole needs <b>%s</b> to play this file.\n"
+                                             "It can be installed automatically.",
+                                             desc);
+    
+    return GTK_DIALOG(dialog);
+}
+
 static gboolean
 parole_gst_bus_event (GstBus *bus, GstMessage *msg, gpointer data)
 {
-    ParoleGst *gst;
+    ParoleGst                *gst;
+    GtkDialog                *dialog;
+    gchar*                    details[2];
+    GstInstallPluginsContext *ctx;
+    gint response;
     
     gst = PAROLE_GST (data);
 
@@ -1131,28 +1472,12 @@ parole_gst_bus_event (GstBus *bus, GstMessage *msg, gpointer data)
         case GST_MESSAGE_EOS:
 	{
 	    ParoleMediaType media_type;
-	    gint current_track;
 	    
 	    TRACE ("End of stream");
 	    
 	    g_object_get (G_OBJECT (gst->priv->stream),
 			  "media-type", &media_type,
 			  NULL);
-	    if ( media_type == PAROLE_MEDIA_TYPE_CDDA )
-	    {
-		gint num_tracks;
-		g_object_get (G_OBJECT (gst->priv->stream),
-			      "num-tracks", &num_tracks,
-			      "track", &current_track,
-			      NULL);
-			  
-		TRACE ("Current track %d Number of tracks %d", current_track, num_tracks);
-		if ( num_tracks != current_track )
-		{
-		    parole_gst_seek_cdda_track (gst, current_track);
-		    break;
-		}
-	    }
 		
 	    gst->priv->media_state = PAROLE_STATE_PLAYBACK_FINISHED;
 	    g_signal_emit (G_OBJECT (gst), signals [MEDIA_STATE], 0, 
@@ -1220,6 +1545,25 @@ parole_gst_bus_event (GstBus *bus, GstMessage *msg, gpointer data)
 	    }
 	    break;
 	case GST_MESSAGE_ELEMENT:
+	    if (gst_is_missing_plugin_message(msg))
+	    {
+	        g_debug("Missing plugin\n");
+	        parole_gst_stop(gst);
+	        dialog = parole_gst_missing_codec_dialog(gst, msg);
+	        response = gtk_dialog_run(dialog);
+	        if ( response == GTK_RESPONSE_ACCEPT )
+	        {
+	             gtk_widget_destroy(GTK_WIDGET(dialog));
+	             details[0] = gst_missing_plugin_message_get_installer_detail(msg);
+	             details[1] = NULL;
+	             ctx = gst_install_plugins_context_new();
+	             gst_install_plugins_async(details, ctx, parole_gst_install_plugins_result_func, gst);
+	             
+	             gst_install_plugins_context_free(ctx);
+	        }
+	        else if ( response == GTK_RESPONSE_REJECT )
+	            gtk_widget_destroy(GTK_WIDGET(dialog));
+        }
 	    break;
 	case GST_MESSAGE_WARNING:
 	    break;
@@ -1324,6 +1668,7 @@ parole_gst_play_file_internal (ParoleGst *gst)
     if ( gst->priv->update_vis)
 	parole_gst_update_vis (gst);
     
+    parole_stream_set_image (G_OBJECT (gst->priv->stream), NULL);
     
     g_object_get (G_OBJECT (gst->priv->stream),
 		  "uri", &uri,
@@ -1341,6 +1686,40 @@ parole_gst_play_file_internal (ParoleGst *gst)
     g_free (uri);
 }
 
+void
+parole_gst_send_navigation_command(ParoleGst *gst, gint command)
+{
+    GstNavigation *nav;
+    nav = GST_NAVIGATION (gst->priv->video_sink);
+    
+    switch (command)
+    {
+        case GST_DVD_ROOT_MENU:
+            TRACE("Root Menu");
+            gst_navigation_send_command (nav, GST_NAVIGATION_COMMAND_DVD_MENU);
+            break;
+        case GST_DVD_TITLE_MENU:
+            TRACE("Title Menu");
+            gst_navigation_send_command (nav, GST_NAVIGATION_COMMAND_DVD_TITLE_MENU);
+            break;
+        case GST_DVD_AUDIO_MENU:
+            TRACE("Audio Menu");
+            gst_navigation_send_command (nav, GST_NAVIGATION_COMMAND_DVD_AUDIO_MENU);
+            break;
+        case GST_DVD_ANGLE_MENU:
+            TRACE("Angle Menu");
+            gst_navigation_send_command (nav, GST_NAVIGATION_COMMAND_DVD_ANGLE_MENU);
+            break;
+        case GST_DVD_CHAPTER_MENU:
+            TRACE("Chapter Menu");
+            gst_navigation_send_command (nav, GST_NAVIGATION_COMMAND_DVD_CHAPTER_MENU);
+            break;
+        default:
+            break;
+    }
+
+}
+
 static gboolean
 parole_gst_motion_notify_event (GtkWidget *widget, GdkEventMotion *ev)
 {
@@ -1376,7 +1755,6 @@ parole_gst_button_press_event (GtkWidget *widget, GdkEventButton *ev)
     {
 	nav = GST_NAVIGATION (gst->priv->video_sink);
 	gst_navigation_send_mouse_event (nav, "mouse-button-press", ev->button, ev->x, ev->y);
-	//ret = TRUE;
     }
     
     if (GTK_WIDGET_CLASS (parole_gst_parent_class)->button_press_event)
@@ -1411,19 +1789,6 @@ parole_gst_button_release_event (GtkWidget *widget, GdkEventButton *ev)
     return ret;
 }
 
-static void     parole_gst_seek_cdda_track	(ParoleGst *gst,
-						 gint track)
-{
-    TRACE ("Track %d", track);
-    
-    if ( !gst_element_seek (gst->priv->playbin, 1.0, gst_format_get_by_nick ("track"), 
-			    GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 
-			    track,
-			    GST_SEEK_TYPE_NONE,
-			    0) )
-	g_warning ("Seek to track %d failed ", track);
-}
-
 static void
 parole_gst_seek_by_format (ParoleGst *gst, GstFormat format, gint step)
 {
@@ -1452,7 +1817,6 @@ parole_gst_change_dvd_chapter (ParoleGst *gst, gint level)
 {
     GstFormat format;
 
-    // FIXME: Do we really need to get the nick each time?
     format = gst_format_get_by_nick ("chapter");
     
     parole_gst_seek_by_format (gst, format, level);
@@ -1653,7 +2017,9 @@ parole_gst_constructed (GObject *object)
     
     gst = PAROLE_GST (object);
     
-    enable_xv = parole_rc_read_entry_bool ("enable-xv", PAROLE_RC_GROUP_GENERAL, TRUE);
+    g_object_get (G_OBJECT (gst->priv->conf),
+		  "enable-xv", &enable_xv,
+		  NULL);
     
     gst->priv->playbin = gst_element_factory_make ("playbin2", "player");
  
@@ -1815,6 +2181,24 @@ parole_gst_class_init (ParoleGstClass *klass)
                       g_cclosure_marshal_VOID__STRING,
                       G_TYPE_NONE, 1, 
 		      G_TYPE_STRING);
+		      
+    signals[DVD_CHAPTER_CHANGE] = 
+        g_signal_new ("dvd-chapter-change",
+                      PAROLE_TYPE_GST,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ParoleGstClass, dvd_chapter_change),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__INT,
+                      G_TYPE_NONE, 1, G_TYPE_INT);
+                      
+    signals[DVD_CHAPTER_COUNT_CHANGE] = 
+        g_signal_new ("dvd-chapter-count-change",
+                      PAROLE_TYPE_GST,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ParoleGstClass, dvd_chapter_count_change),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__INT,
+                      G_TYPE_NONE, 1, G_TYPE_INT);
 
     g_object_class_install_property (object_class,
 				     PROP_CONF_OBJ,
@@ -1856,6 +2240,8 @@ parole_gst_init (ParoleGst *gst)
     gst->priv->with_vis = FALSE;
     gst->priv->vis_loaded = FALSE;
     gst->priv->scale_logo = TRUE;
+    gst->priv->use_custom_subtitles = FALSE;
+    gst->priv->custom_subtitles = NULL;
     
     gst->priv->conf = NULL;
     
@@ -1917,6 +2303,31 @@ parole_gst_play_idle (gpointer data)
     return FALSE;
 }
 
+void parole_gst_set_custom_subtitles (ParoleGst *gst, const gchar* sub_file)
+{
+    if ( sub_file == NULL )
+    {
+        gst->priv->use_custom_subtitles = FALSE;
+        gst->priv->custom_subtitles = NULL;
+    }
+    else
+    {
+        gst->priv->use_custom_subtitles = TRUE;
+        gst->priv->custom_subtitles = g_strdup(sub_file);
+    }
+}
+
+gchar * parole_gst_get_file_uri (ParoleGst *gst)
+{
+    gchar* uri;
+
+    g_object_get (G_OBJECT (gst->priv->stream),
+          "uri", &uri,
+          NULL);
+          
+    return uri;
+}
+
 void parole_gst_play_uri (ParoleGst *gst, const gchar *uri, const gchar *subtitles)
 {
     g_mutex_lock (&gst->priv->lock);
@@ -1928,7 +2339,7 @@ void parole_gst_play_uri (ParoleGst *gst, const gchar *uri, const gchar *subtitl
 	          "uri", uri,
 		  "subtitles", subtitles,
 		  NULL);
-
+		  
     g_mutex_unlock (&gst->priv->lock);
     
     if ( gst->priv->state_change_id == 0 )
@@ -2114,37 +2525,27 @@ void parole_gst_prev_dvd_chapter (ParoleGst *gst)
     parole_gst_change_dvd_chapter (gst, -1);
 }
 
-void parole_gst_next_cdda_track (ParoleGst *gst)
+void parole_gst_set_dvd_chapter (ParoleGst *gst, gint chapter)
 {
-    //parole_gst_change_cdda_track (gst, 1);
+    GstFormat format = gst_format_get_by_nick ("chapter");
+    guint64 val = (guint64) chapter;
+    
+    gst_element_seek (gst->priv->playbin, 1.0, format, 
+				GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 
+			        val,
+			        GST_SEEK_TYPE_NONE,
+			        0);
+}
 
-    gint num_tracks, current_track;
+gint parole_gst_get_num_tracks (ParoleGst *gst)
+{
+    gint num_tracks;
     
     g_object_get (G_OBJECT (gst->priv->stream),
 			      "num-tracks", &num_tracks,
-			      "track", &current_track,
 			      NULL);
-
-    if ( num_tracks != current_track )
-    {
-        parole_gst_seek_cdda_track (gst, current_track);
-    }
-}
-
-void parole_gst_prev_cdda_track (ParoleGst *gst)
-{
-    //parole_gst_change_cdda_track (gst, -1);
-    
-    gint current_track;
-    
-    g_object_get (G_OBJECT (gst->priv->stream),
-			      "track", &current_track,
-			      NULL);
-
-    if ( current_track != 1 )
-    {
-        parole_gst_seek_cdda_track (gst, current_track-2);
-    }
+			      
+    return num_tracks;
 }
 
 void parole_gst_seek_cdda	(ParoleGst *gst, guint track_num)
@@ -2252,40 +2653,66 @@ gst_get_lang_list_for_type (ParoleGst * gst, const gchar * type_name)
     gint i, n = 0;
 
     g_object_get (G_OBJECT (gst->priv->playbin), "n-text", &n, NULL);
-    if (n == 0)
+    
+    if (n == 0 && gst->priv->use_custom_subtitles == FALSE)
       return NULL;
+      
+    if ( gst->priv->use_custom_subtitles == TRUE )
+        n--;
 
-    for (i = 0; i < n; i++) {
-      GstTagList *tags = NULL;
+    if (n != 0)
+    {
+        for (i = 0; i < n; i++) {
+          GstTagList *tags = NULL;
 
-      g_signal_emit_by_name (G_OBJECT (gst->priv->playbin), "get-text-tags",
-          i, &tags);
+          g_signal_emit_by_name (G_OBJECT (gst->priv->playbin), "get-text-tags",
+              i, &tags);
 
-      if (tags) {
-        gchar *lc = NULL, *cd = NULL;
+          if (tags) {
+            gchar *lc = NULL, *cd = NULL;
 
-	gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE, &lc);
-	gst_tag_list_get_string (tags, GST_TAG_CODEC, &lc);
+	    gst_tag_list_get_string (tags, GST_TAG_LANGUAGE_CODE, &lc);
+	    gst_tag_list_get_string (tags, GST_TAG_CODEC, &lc);
 	
-        if (lc) {
-	  ret = g_list_prepend (ret, lc);
-	  g_free (cd);
-	} else if (cd) {
-	  ret = g_list_prepend (ret, cd);
-	} else {
-	  ret = g_list_prepend (ret, g_strdup_printf (_("Subtitle #%d"), num++));
-	}
-	gst_tag_list_free (tags);
-      } else {
-	ret = g_list_prepend (ret, g_strdup_printf (_("Subtitle #%d"), num++));
-      }
+            if (lc) {
+	      ret = g_list_prepend (ret, lc);
+	      g_free (cd);
+	    } else if (cd) {
+	      ret = g_list_prepend (ret, cd);
+	    } else {
+	      ret = g_list_prepend (ret, g_strdup_printf (_("Subtitle #%d"), num++));
+	    }
+	    gst_tag_list_free (tags);
+          } else {
+	    ret = g_list_prepend (ret, g_strdup_printf (_("Subtitle #%d"), num++));
+          }
+        }
+    }
+    
+    ret = g_list_reverse (ret);
+    
+    if ( gst->priv->use_custom_subtitles == TRUE )
+    {
+        ret = g_list_prepend (ret, g_strdup_printf("%s",gst->priv->custom_subtitles));
     }
   } else {
     g_critical ("Invalid stream type '%s'", type_name);
     return NULL;
   }
 
-  return g_list_reverse (ret);
+  return ret;
+}
+
+gboolean
+gst_get_has_vis( ParoleGst *gst )
+{
+    gboolean has_vis;
+    
+    g_object_get (G_OBJECT (gst->priv->conf),
+		  "vis-enabled", &has_vis,
+		  NULL);
+		  
+    return has_vis;
 }
 
 gboolean
@@ -2317,7 +2744,10 @@ gst_set_current_subtitle_track( ParoleGst *gst, gint track_no )
 		  "uri", &uri,
 		  NULL);
 		  
-	sub = (gchar*) parole_get_subtitle_path(uri);
+    if ( gst->priv->use_custom_subtitles == TRUE )
+        sub = gst->priv->custom_subtitles;
+    else
+	    sub = (gchar*) parole_get_subtitle_path(uri);
 
 	g_object_get (gst->priv->playbin, "flags", &flags, NULL);
 	
