@@ -45,8 +45,10 @@ struct _ParoleFilePrivate
     gchar 	*filename;
     gchar 	*display_name;
     gchar 	*uri;
-    gchar       *content_type;
+    gchar   *content_type;
 	gchar	*directory;
+	gchar   *custom_subtitles;
+	gint    dvd_chapter;
     
 };
 
@@ -57,7 +59,9 @@ enum
     PROP_DISPLAY_NAME,
     PROP_URI,
     PROP_CONTENT_TYPE,
-	PROP_DIRECTORY
+	PROP_DIRECTORY,
+	PROP_CUSTOM_SUBTITLES,
+	PROP_DVD_CHAPTER
 };
 
 G_DEFINE_TYPE (ParoleFile, parole_file, G_TYPE_OBJECT)
@@ -85,6 +89,9 @@ parole_file_finalize (GObject *object)
 	
 	if ( priv->directory )
 	g_free (priv->directory);
+	
+	if ( priv->custom_subtitles )
+	g_free (priv->custom_subtitles);
     
     G_OBJECT_CLASS (parole_file_parent_class)->finalize (object);
 }
@@ -107,6 +114,12 @@ parole_file_set_property (GObject *object, guint prop_id,
 	case PROP_DIRECTORY:
 		PAROLE_FILE_GET_PRIVATE (file)->directory = g_value_dup_string (value);
 		break;
+	case PROP_CUSTOM_SUBTITLES:
+	    PAROLE_FILE_GET_PRIVATE (file)->custom_subtitles = g_value_dup_string (value);
+	    break;
+    case PROP_DVD_CHAPTER:
+	    PAROLE_FILE_GET_PRIVATE (file)->dvd_chapter = g_value_get_int (value);
+	    break;
 	default:
 	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	    break;
@@ -138,6 +151,12 @@ parole_file_get_property (GObject *object, guint prop_id,
 	case PROP_DIRECTORY:
 	    g_value_set_string (value, PAROLE_FILE_GET_PRIVATE (file)->directory);
 	    break;
+    case PROP_CUSTOM_SUBTITLES:
+        g_value_set_string (value, PAROLE_FILE_GET_PRIVATE (file)->custom_subtitles);
+        break;
+    case PROP_DVD_CHAPTER:
+	    g_value_set_int (value, PAROLE_FILE_GET_PRIVATE (file)->dvd_chapter);
+	    break;
 	default:
 	    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	    break;
@@ -153,9 +172,33 @@ parole_file_constructed (GObject *object)
     ParoleFilePrivate *priv;
     GError *error = NULL;
     
+    gchar *filename;
+    
     file = PAROLE_FILE (object);
     priv = PAROLE_FILE_GET_PRIVATE (file);
     
+    filename = g_strdup(priv->filename);
+    
+    if ( g_str_has_prefix(filename, "cdda") )
+    {
+        priv->directory = NULL;
+        priv->uri = g_strdup(filename);
+        priv->content_type = g_strdup("cdda");
+        g_free(filename);
+        return;
+    }
+    
+    if ( g_str_has_prefix(filename, "dvd") )
+    {
+        priv->directory = NULL;
+        priv->uri = "dvd:/";
+        priv->content_type = g_strdup("dvd");
+        g_free(filename);
+        return;
+    }
+    
+    g_free(filename);
+    
     gfile = g_file_new_for_commandline_arg (priv->filename);
 
     info = g_file_query_info (gfile, 
@@ -299,7 +342,7 @@ parole_file_class_init (ParoleFileClass *klass)
 							  "Content type", 
 							  "The content type of the file",
 							  NULL,
-							  G_PARAM_READABLE));
+  							  G_PARAM_READABLE));
 							  
 	/**
      * ParoleFile:directory:
@@ -316,6 +359,40 @@ parole_file_class_init (ParoleFileClass *klass)
 							  NULL,
 							  G_PARAM_CONSTRUCT_ONLY|
 							  G_PARAM_READWRITE));
+							  
+    /**
+     * ParoleFile:custom_subtitles:
+     *
+     * The custom subtitles set by the user.
+     *
+     * Since: 0.4
+     **/
+    g_object_class_install_property (object_class,
+				     PROP_CUSTOM_SUBTITLES,
+				     g_param_spec_string ("custom_subtitles",
+							  "Custom Subtitles", 
+							  "The custom subtitles set by the user",
+							  NULL,
+							  G_PARAM_CONSTRUCT_ONLY|
+							  G_PARAM_READWRITE));
+							  
+    /**
+     * ParoleFile:dvd_chapter:
+     *
+     * DVD Chapter, used for seeking a DVD using the playlist.
+     *
+     * Since: 0.4
+     **/
+    g_object_class_install_property (object_class,
+				     PROP_DVD_CHAPTER,
+				     g_param_spec_int ("dvd-chapter",
+							  "DVD Chapter", 
+							  "DVD Chapter, used for seeking a DVD using the playlist.",
+							  -1,
+							  1000,
+							  -1,
+							  G_PARAM_CONSTRUCT_ONLY|
+							  G_PARAM_READWRITE));
 
     g_type_class_add_private (klass, sizeof (ParoleFilePrivate));
 }
@@ -332,6 +409,8 @@ parole_file_init (ParoleFile *file)
     priv->uri          = NULL;
     priv->content_type    = NULL;
 	priv->directory			= NULL;
+	priv->custom_subtitles = NULL;
+	priv->dvd_chapter = 0;
 }
 
 /**
@@ -366,10 +445,63 @@ ParoleFile *
 parole_file_new_with_display_name (const gchar *filename, const gchar *display_name)
 {
     ParoleFile *file = NULL;
+    
     file = g_object_new (PAROLE_TYPE_FILE, 
 			 "filename", filename, 
 			 "display-name", display_name, 
 			 NULL);
+
+    return file;
+}
+
+/**
+ * parole_file_new_cdda_track:
+ * @track_num: cd track number.
+ * 
+ * 
+ * 
+ * Returns: A new #ParoleFile object.
+ * 
+ * Since: 0.4
+ **/
+ParoleFile *
+parole_file_new_cdda_track (const gint track_num, const gchar *display_name)
+{
+    ParoleFile *file = NULL;
+    gchar *uri = g_strdup_printf("cdda://%i", track_num);
+
+    file = g_object_new (PAROLE_TYPE_FILE, 
+		 "filename", uri, 
+		 "display-name", display_name, 
+		 NULL);
+    
+    g_free(uri);
+    return file;
+}
+
+/**
+ * parole_file_new_dvd_chapter:
+ * @track_num: dvd chapter number.
+ * 
+ * 
+ * 
+ * Returns: A new #ParoleFile object.
+ * 
+ * Since: 0.4
+ **/
+ParoleFile *
+parole_file_new_dvd_chapter (gint chapter_num, const gchar *display_name)
+{
+    ParoleFile *file = NULL;
+    gchar *uri = "dvd:/";
+
+    file = g_object_new (PAROLE_TYPE_FILE, 
+		 "filename", uri, 
+		 "display-name", display_name, 
+		 "dvd-chapter", chapter_num,
+		 NULL);
+    
+    //g_free(uri); FIXME This should probably be freed.
     return file;
 }
 
@@ -457,3 +589,72 @@ parole_file_get_directory (const ParoleFile *file)
     
     return PAROLE_FILE_GET_PRIVATE (file)->directory;
 }
+
+/**
+ * parole_file_get_custom_subtitles:
+ * @file: a #ParoleFile.
+ *
+ *
+ * Returns: A string containing the custom subtitles file path.
+ *
+ * Since: 0.4
+ **/
+const gchar *
+parole_file_get_custom_subtitles (const ParoleFile *file)
+{
+    g_return_val_if_fail (PAROLE_IS_FILE (file), NULL);
+    
+    return PAROLE_FILE_GET_PRIVATE (file)->custom_subtitles;
+}
+
+void
+parole_file_set_custom_subtitles (const ParoleFile *file, gchar *suburi)
+{
+    GValue value = G_VALUE_INIT;
+    g_value_init (&value, G_TYPE_STRING);
+    g_value_set_static_string (&value, suburi);
+    
+    parole_file_set_property (G_OBJECT(file), PROP_CUSTOM_SUBTITLES, 
+			      &value, g_param_spec_string ("custom_subtitles",
+							  "Custom Subtitles", 
+							  "The custom subtitles set by the user",
+							  NULL,
+							  G_PARAM_CONSTRUCT_ONLY|
+							  G_PARAM_READWRITE));
+}
+
+/**
+ * parole_file_get_dvd_chapter:
+ * @file: a #ParoleFile.
+ *
+ *
+ * Returns: An int containing the dvd chapter number.
+ *
+ * Since: 0.4
+ **/
+gint
+parole_file_get_dvd_chapter (const ParoleFile *file)
+{
+    //g_return_val_if_fail (PAROLE_IS_FILE (file), NULL);
+    
+    return PAROLE_FILE_GET_PRIVATE (file)->dvd_chapter;
+}
+
+void
+parole_file_set_dvd_chapter (const ParoleFile *file, gint dvd_chapter)
+{
+    GValue value = G_VALUE_INIT;
+    g_value_init (&value, G_TYPE_INT);
+    g_value_set_int (&value, dvd_chapter);
+    
+    parole_file_set_property (G_OBJECT(file), PROP_DVD_CHAPTER, 
+			      &value, g_param_spec_int ("dvd-chapter",
+							  "DVD Chapter", 
+							  "DVD Chapter, used for seeking a DVD using the playlist.",
+							  -1,
+							  1000,
+							  -1,
+							  G_PARAM_CONSTRUCT_ONLY|
+							  G_PARAM_READWRITE));
+}
+
