@@ -68,6 +68,7 @@ struct _ParoleStreamPrivate
     gchar      *year;
     gchar      *album;
     gchar      *comment;
+    GdkPixbuf  *image;
     
     ParoleMediaType media_type; 
 };
@@ -311,6 +312,38 @@ parole_stream_finalize (GObject *object)
     G_OBJECT_CLASS (parole_stream_parent_class)->finalize (object);
 }
 
+void 
+parole_stream_set_image (GObject *object, GdkPixbuf *pixbuf)
+{
+    ParoleStream *stream;
+    
+    stream = PAROLE_STREAM (object);
+    
+    if ( PAROLE_STREAM_GET_PRIVATE (stream)->image )
+    g_object_unref(G_OBJECT(PAROLE_STREAM_GET_PRIVATE (stream)->image));
+    
+    if (pixbuf)
+    PAROLE_STREAM_GET_PRIVATE (stream)->image = gdk_pixbuf_copy(pixbuf);
+    else
+    PAROLE_STREAM_GET_PRIVATE (stream)->image = NULL;
+}
+
+GdkPixbuf *
+parole_stream_get_image (GObject *object)
+{
+    ParoleStream *stream;
+    GdkPixbuf *pixbuf;
+    
+    stream = PAROLE_STREAM (object);
+    
+    if (PAROLE_STREAM_GET_PRIVATE (stream)->image)
+    pixbuf = gdk_pixbuf_copy(GDK_PIXBUF(PAROLE_STREAM_GET_PRIVATE (stream)->image));
+    else
+    pixbuf = NULL;
+    
+    return pixbuf;
+}
+
 static void
 parole_stream_class_init (ParoleStreamClass *klass)
 {
@@ -560,7 +593,8 @@ parole_stream_class_init (ParoleStreamClass *klass)
      * ParoleStream:track:
      * 
      * Currently playing track, this is only valid if
-     * #ParoleStream:media-type: is PAROLE_MEDIA_TYPE_CDDA.
+     * #ParoleStream:media-type: is PAROLE_MEDIA_TYPE_CDDA 
+     *                           or PAROLE_MEDIA_TYPE_DVD.
      * 
      * Since: 0.2 
      **/
@@ -569,7 +603,7 @@ parole_stream_class_init (ParoleStreamClass *klass)
 				     g_param_spec_uint   ("track",
 							  "Track", 
 							  "Track",
-							  1, 99,
+							  0, 99,
 							  1,
 							  G_PARAM_READWRITE));
     /**
