@@ -62,6 +62,7 @@ struct ParoleDiscPrivate
 enum
 {
     DISC_SELECTED,
+    LABEL_CHANGED,
     LAST_SIGNAL
 };
 
@@ -78,6 +79,12 @@ typedef struct
     
 } MountData;
 
+/**
+ * free_mount_data:
+ * @data : pointer to the mount point.
+ *
+ * Free the mount-point.
+ **/
 static void
 free_mount_data (gpointer data)
 {
@@ -94,22 +101,47 @@ free_mount_data (gpointer data)
     g_free (mount);
 }
 
+
 static void
-parole_disc_media_activate_cb (GtkWidget *widget, ParoleDisc *disc)
+parole_disc_set_label(ParoleDisc *disc, const gchar *label)
 {
-    MountData *data;
+    gchar *menu_label;
     
-    data = g_object_get_data (G_OBJECT (widget), "mount-data");
+    if ( g_strcmp0(label, _("Insert Disc")) != 0 )
+    {
+    menu_label = g_strdup_printf ("%s '%s'", _("Play Disc"), label);
+    g_signal_emit (G_OBJECT (disc), signals [LABEL_CHANGED], 0, label);
+    }
+    else
+    {
+    menu_label = g_strdup(label);
+    g_signal_emit (G_OBJECT (disc), signals [LABEL_CHANGED], 0, label);
+    }
     
-    g_signal_emit (G_OBJECT (disc), signals [DISC_SELECTED], 0, data->uri, data->device);
+    gtk_menu_item_set_label( GTK_MENU_ITEM (disc->priv->disc_menu_item), menu_label );
+    g_free(menu_label);
 }
 
 static void
-parole_disc_show_menu_item (ParoleDisc *disc, MountData *data, const gchar *label)
+parole_disc_set_enabled(ParoleDisc *disc, gboolean enabled)
 {
-	GtkWidget *img;
-	
-	switch (data->kind)
+    gtk_widget_set_sensitive( GTK_WIDGET(disc->priv->disc_menu_item), enabled);
+    //g_signal_emit (G_OBJECT (disc), signals [DVD_ENABLED], 0, enabled);
+}
+
+static gboolean
+parole_disc_get_enabled(ParoleDisc *disc)
+{
+    return gtk_widget_get_sensitive( GTK_WIDGET(disc->priv->disc_menu_item) );
+}
+
+static void
+parole_disc_set_kind(ParoleDisc *disc, ParoleDiscKind kind)
+{
+    GtkWidget *img;
+    gboolean enabled = TRUE;
+    
+    switch (kind)
 	{
 		case PAROLE_DISC_CDDA:
 			img = gtk_image_new_from_icon_name("media-cdrom-audio", GTK_ICON_SIZE_MENU);
@@ -121,33 +153,69 @@ parole_disc_show_menu_item (ParoleDisc *disc, MountData *data, const gchar *labe
 			break;
 		default:
 		    img = gtk_image_new_from_stock("gtk-cdrom", GTK_ICON_SIZE_MENU);
-		    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM (disc->priv->disc_menu_item), img);
-		    gtk_menu_item_set_label( GTK_MENU_ITEM (disc->priv->disc_menu_item), _("Insert Disc") );
-		    gtk_widget_set_sensitive( GTK_WIDGET (disc->priv->disc_menu_item), FALSE );
-		    data->mi = disc->priv->disc_menu_item;
-	        gtk_widget_show (data->mi);
-            gtk_widget_show (img);
-		    return;
+		    parole_disc_set_label(disc, _("Insert Disc") );
+		    enabled = FALSE;
 		    break;
 	}
 	
 	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM (disc->priv->disc_menu_item), img);
-	gtk_menu_item_set_label( GTK_MENU_ITEM (disc->priv->disc_menu_item), label );
-	
-	gtk_widget_set_sensitive( GTK_WIDGET (disc->priv->disc_menu_item), TRUE );
-
-	data->mi = disc->priv->disc_menu_item;
-	
-	gtk_widget_show (data->mi);
-    gtk_widget_show (img);
-	
-    g_object_set_data (G_OBJECT (data->mi),
-		      "mount-data", data);
-		      
-	g_signal_connect (data->mi, "activate",
-		      G_CALLBACK (parole_disc_media_activate_cb), disc);
+	parole_disc_set_enabled(disc, enabled);
 }
 
+
+/**
+ * parole_disc_media_activate_cb:
+ * @widget : the #GtkWidget activated for this callback function.
+ * @disc   : the #ParoleDisc instance.
+ *
+ * Callback function for when the CD/DVD menu item is activated.
+ **/
+static void
+parole_disc_media_activate_cb (GtkWidget *widget, ParoleDisc *disc)
+{
+    MountData *data;
+    
+    data = g_object_get_data (G_OBJECT (widget), "mount-data");
+    
+    g_signal_emit (G_OBJECT (disc), signals [DISC_SELECTED], 0, data->uri, data->device);
+}
+
+/**
+ * parole_disc_show_menu_item:
+ * @disc  : the #ParoleDisc instance.
+ * @data  : the #MountData of the inserted disc.
+ * @label : the name of the inserted disc.
+ * 
+ * Show the respective disc-item in the "Media" menu, or "Insert Disc" if no 
+ * disc is detected.
+ **/
+static void
+parole_disc_show_menu_item (ParoleDisc *disc, MountData *data, const gchar *label)
+{
+	parole_disc_set_kind (disc, data->kind);
+	parole_disc_set_label(disc, label);
+
+	if (parole_disc_get_enabled(disc))
+	{
+	    data->mi = disc->priv->disc_menu_item;
+	    
+        g_object_set_data (G_OBJECT (data->mi),
+		          "mount-data", data);
+		          
+	    g_signal_connect (data->mi, "activate",
+		          G_CALLBACK (parole_disc_media_activate_cb), disc);
+    }
+}
+
+/**
+ * parole_disc_get_mount_data:
+ * @disc   : the #ParoleDisc instance.
+ * @uri    : the URI of the mount point.
+ * @device : the device identifier of the mount point.
+ * @kind   : the #ParoleDiscKind representing the type of disc (CD/DVD/SVCD).
+ *
+ * Get data from the mount-point.
+ **/
 static MountData *
 parole_disc_get_mount_data (ParoleDisc *disc, 
 			    const gchar *uri, 
@@ -166,6 +234,14 @@ parole_disc_get_mount_data (ParoleDisc *disc,
     return data;
 }
 
+/**
+ * parole_disc_add_mount_to_menu:
+ * @disc   : the #ParoleDisc instance.
+ * @mount  : the #GMount representing the mounted disc.
+ * @device : the device identifier of the mount point.
+ *
+ * Add the mounted disc to the "Media" menu.
+ **/
 static void
 parole_disc_add_mount_to_menu (ParoleDisc *disc, GMount *mount, const gchar *device)
 {
@@ -183,9 +259,17 @@ parole_disc_add_mount_to_menu (ParoleDisc *disc, GMount *mount, const gchar *dev
 	uri = g_strdup ("cdda://");
 	goto got_cdda;
     }
+    
+    if ( g_file_has_uri_scheme (file, "dvd") )
+    {
+	kind = PAROLE_DISC_DVD;
+	uri = g_strdup ("dvd:/");
+	goto got_cdda;
+    }
 	
     content_type = g_content_type_guess_for_tree (file);
 
+    /* Determine the type of disc */
     for ( i = 0; content_type && content_type[i]; i++)
     {
 	TRACE ("Checking disc content type : %s", content_type[i]);
@@ -224,25 +308,30 @@ got_cdda:
     {
 	MountData *data;
 	gchar *name;
-	gchar *label;
 	
 	name = g_mount_get_name (mount);
-	label = g_strdup_printf ("%s '%s'", _("Play Disc"), name);
 	
 	data = parole_disc_get_mount_data (disc, uri, device, kind);
-	parole_disc_show_menu_item (disc, data, label);
+	parole_disc_show_menu_item (disc, data, name);
 	
 	if ( uri )
 	    g_free (uri);
 	
 	g_ptr_array_add (disc->priv->array, data);
-	g_free (label);
 	g_free (name);
     }
     
     g_object_unref (file);
 }
 
+/**
+ * parole_disc_check_cdrom:
+ * @disc   : the #ParoleDisc instance.
+ * @volume : the #GVolume for the mounted disc.
+ * @device : the device identifier of the mount point.
+ *
+ * Check the state of the drive.
+ **/
 static void
 parole_disc_check_cdrom (ParoleDisc *disc, GVolume *volume, const gchar *device)
 {
@@ -253,7 +342,6 @@ parole_disc_check_cdrom (ParoleDisc *disc, GVolume *volume, const gchar *device)
     MountData *data;
     
 	gchar *name;
-	gchar *label;
 
     TRACE ("device : %s", device);
     
@@ -280,9 +368,8 @@ parole_disc_check_cdrom (ParoleDisc *disc, GVolume *volume, const gchar *device)
 		    data = parole_disc_get_mount_data (disc, "cdda://", device, PAROLE_DISC_CDDA);
 	
 			name = g_volume_get_name (volume);
-			label = g_strdup_printf ("%s '%s'", _("Play Disc"), name);
 		    
-		    parole_disc_show_menu_item (disc, data, label);
+		    parole_disc_show_menu_item (disc, data, name);
 		    g_ptr_array_add (disc->priv->array, data);
 		}
 	    }
@@ -295,6 +382,14 @@ out:
 #endif /* if defined(__linux__) */
 }
 
+/**
+ * parole_disc_add_drive:
+ * @disc   : the #ParoleDisc instance.
+ * @drive  : the #GDrive for the mounted disc.
+ * @device : the device identifier of the mount point.
+ *
+ * Add the disc drive to the menu.
+ **/
 static void
 parole_disc_add_drive (ParoleDisc *disc, GDrive *drive, const gchar *device)
 {
@@ -331,6 +426,12 @@ parole_disc_add_drive (ParoleDisc *disc, GDrive *drive, const gchar *device)
     g_list_free (list);
 }
 
+/**
+ * parole_disc_get_drives:
+ * @disc : the #ParoleDisc instance.
+ * 
+ * Get the list of available drives and attempt to add each to the menu.
+ **/
 static void
 parole_disc_get_drives (ParoleDisc *disc)
 {
@@ -355,6 +456,7 @@ parole_disc_get_drives (ParoleDisc *disc)
 	
 	drive = g_list_nth_data (list, i);
 	
+	/* FIXME what happens if there is more than one disc drive? */
 	if ( g_drive_can_eject (drive) && g_drive_has_media (drive) )
 	{
 	    device = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
@@ -368,6 +470,14 @@ parole_disc_get_drives (ParoleDisc *disc)
     g_list_free (list);
 }
 
+/**
+ * parole_disc_select_cb:
+ * @item : "Media" menu item passed to the callback function.
+ * @disc : the #ParoleDisc instance.
+ *
+ * Callback function for selecting the "Media" menu item.  If a disc update is
+ * needed, perform it when the menu item is activated.
+ **/
 static void
 parole_disc_select_cb (GtkItem *item, ParoleDisc *disc)
 {
@@ -375,19 +485,29 @@ parole_disc_select_cb (GtkItem *item, ParoleDisc *disc)
 	parole_disc_get_drives (disc);
 }
 
+/**
+ * parole_disc_monitor_changed_cb:
+ * @monitor : the #GVolumeMonitor that monitors changes to attached volumes.
+ * @device  : the device identifier of the mount point.
+ * @disc    : the #ParoleDisc instance.
+ *
+ * Callback function for when attached volumes are modified.  Set the disc item
+ * to blank and tell the #ParoleDisc to check for updates.
+ **/
 static void
 parole_disc_monitor_changed_cb (GVolumeMonitor *monitor, gpointer *device, ParoleDisc *disc)
 {
-    GtkWidget *img;
-	
-	img = gtk_image_new_from_stock("gtk-cdrom", GTK_ICON_SIZE_MENU);
-    gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM (disc->priv->disc_menu_item), img);
-    gtk_menu_item_set_label( GTK_MENU_ITEM (disc->priv->disc_menu_item), _("Insert Disc") );
-    gtk_widget_set_sensitive( GTK_WIDGET (disc->priv->disc_menu_item), FALSE );
+    parole_disc_set_kind(disc, PAROLE_DISC_UNKNOWN);
     
     disc->priv->needs_update = TRUE;
 }
 
+/**
+ * parole_disc_class_init:
+ * @klass : the #ParoleDiscClass to initialize.
+ *
+ * Initialize the #ParoleDiscClass.
+ **/
 static void
 parole_disc_class_init (ParoleDiscClass *klass)
 {
@@ -402,12 +522,27 @@ parole_disc_class_init (ParoleDiscClass *klass)
 		      _gmarshal_VOID__STRING_STRING,
                       G_TYPE_NONE, 2, 
 		      G_TYPE_STRING, G_TYPE_STRING);
+
+    signals[LABEL_CHANGED] = 
+        g_signal_new ("label-changed",
+                      PAROLE_TYPE_DISC,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ParoleDiscClass, label_changed),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__STRING,
+                      G_TYPE_NONE, 1, G_TYPE_STRING);
 		      
     object_class->finalize = parole_disc_finalize;
 
     g_type_class_add_private (klass, sizeof (ParoleDiscPrivate));
 }
 
+/**
+ * parole_disc_init:
+ * @disc : the #ParoleDisc to initialize.
+ * 
+ * Initialize the disc monitor.
+ **/
 static void
 parole_disc_init (ParoleDisc *disc)
 {
@@ -422,6 +557,7 @@ parole_disc_init (ParoleDisc *disc)
     
     disc->priv->monitor = g_volume_monitor_get ();
     
+    /* Connect the various disc signals */
     g_signal_connect (G_OBJECT (disc->priv->monitor), "volume-added",
 		      G_CALLBACK (parole_disc_monitor_changed_cb), disc);
     
@@ -448,6 +584,12 @@ parole_disc_init (ParoleDisc *disc)
     g_object_unref (builder);
 }
 
+/**
+ * parole_disc_finalize:
+ * @object : a base #GObject to be made into a #ParoleDisc object.
+ *
+ * Finalize a #ParoleDisc object.
+ **/
 static void
 parole_disc_finalize (GObject *object)
 {
@@ -463,6 +605,11 @@ parole_disc_finalize (GObject *object)
     G_OBJECT_CLASS (parole_disc_parent_class)->finalize (object);
 }
 
+/**
+ * parole_disc_new:
+ *
+ * Create a new #ParoleDisc instance.
+ **/
 ParoleDisc *
 parole_disc_new (void)
 {
