@@ -51,7 +51,6 @@
 #include "parole-filters.h"
 #include "parole-pl-parser.h"
 #include "parole-utils.h"
-#include "parole-rc-utils.h"
 #include "parole-dbus.h"
 
 #include "common/parole-common.h"
@@ -66,6 +65,7 @@ typedef struct
     
 } ParolePlaylistSave;
 
+/* Playlist filetypes */
 static struct
 {
     ParolePlFormat format;
@@ -92,11 +92,9 @@ static GtkTreeRowReference *
 						   GtkTreeIter *iter, 
 						   gboolean select_path);
 
-static void 	parole_media_list_select_path 	  (ParoleMediaList *list, 
+static void 	parole_media_list_select_path 	  (ParoleMediaList *list, gboolean disc,
 						   GtkTreePath *path);
 
-static void	parole_media_list_clear_list 	  (ParoleMediaList *list);
-
 /*
  * Callbacks for GtkBuilder
  */
@@ -158,6 +156,18 @@ gboolean	parole_media_list_query_tooltip		(GtkWidget *widget,
 							 
 void		parole_media_list_menu_pos (GtkMenu *menu, 
 							gint *px, gint *py, gpointer data);
+
+void parole_media_list_send_dvd_menu_navigation (ParoleMediaList *list, gint msg_id);
+
+void parole_media_list_dvd_menu_activated (GtkMenuItem *widget, ParoleMediaList *list);
+
+void parole_media_list_dvd_title_activated (GtkMenuItem *widget, ParoleMediaList *list);
+
+void parole_media_list_dvd_audio_activated (GtkMenuItem *widget, ParoleMediaList *list);
+
+void parole_media_list_dvd_angle_activated (GtkMenuItem *widget, ParoleMediaList *list);
+
+void parole_media_list_dvd_chapter_activated (GtkMenuItem *widget, ParoleMediaList *list);
 							 
 /*
  * End of GtkBuilder callbacks
@@ -170,10 +180,20 @@ struct ParoleMediaListPrivate
 {
     DBusGConnection     *bus;
     ParoleConf          *conf;
-    GtkWidget 	  	*view;
-    GtkWidget		*box;
-    GtkListStore	*store;
+    GtkWidget 	  	    *view;
+    GtkWidget           *disc_view;
+    GtkWidget		    *box;
+    GtkListStore	    *store;
+    GtkListStore	    *disc_store;
     GtkTreeSelection    *sel;
+    GtkTreeSelection    *disc_sel;
+    
+    GtkWidget *dvd_menu;
+    GtkWidget *dvd_menu_button;
+    GtkWidget *dvd_label;
+    GtkWidget *playlist_controls;
+    
+    GtkWidget *playlist_notebook;
 
     GtkWidget *remove_button;
     GtkWidget *clear_button;
@@ -189,9 +209,20 @@ enum
     URI_OPENED,
     SHUFFLE_TOGGLED,
     REPEAT_TOGGLED,
+    SHOW_PLAYLIST,
+    GST_DVD_NAV_MESSAGE,
     LAST_SIGNAL
 };
 
+enum
+{
+    GST_DVD_ROOT_MENU,
+    GST_DVD_TITLE_MENU,
+    GST_DVD_AUDIO_MENU,
+    GST_DVD_ANGLE_MENU,
+    GST_DVD_CHAPTER_MENU
+};
+
 static guint signals [LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (ParoleMediaList, parole_media_list, GTK_TYPE_VBOX)
@@ -207,6 +238,7 @@ parole_media_list_set_widget_sensitive (ParoleMediaList *list, gboolean sensitiv
  * parole_media_list_add:
  * @ParoleMediaList: a #ParoleMediaList
  * @file: a #ParoleFile
+ * @disc: TRUE if added to disc playlist.
  * @emit: TRUE to emit a play signal.
  * @select_row: TRUE to select the added row
  * 
@@ -215,7 +247,7 @@ parole_media_list_set_widget_sensitive (ParoleMediaList *list, gboolean sensitiv
  * start playing the added file.
  **/
 static void
-parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean emit, gboolean select_row)
+parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean disc, gboolean emit, gboolean select_row)
 {
     GtkListStore *list_store;
     GtkTreePath *path;
@@ -223,7 +255,10 @@ parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean emit, g
     GtkTreeIter iter;
     gint nch;
     
-    list_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (list->priv->view)));
+    if (disc)
+        list_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (list->priv->disc_view)));
+    else
+        list_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (list->priv->view)));
     
     gtk_list_store_append (list_store, &iter);
     
@@ -240,7 +275,7 @@ parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean emit, g
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (list_store), &iter);
 	row = gtk_tree_row_reference_new (GTK_TREE_MODEL (list_store), path);
 	if ( select_row )
-	    parole_media_list_select_path (list, path);
+	    parole_media_list_select_path (list, disc, path);
 	gtk_tree_path_free (path);
 	if ( emit )
 	    g_signal_emit (G_OBJECT (list), signals [MEDIA_ACTIVATED], 0, row);
@@ -253,6 +288,9 @@ parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean emit, g
      */
     g_object_unref (file);
     
+    if (disc)
+    nch = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (list->priv->disc_store), NULL); 
+    else
     nch = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (list->priv->store), NULL); 
     
     if ( nch == 1 )
@@ -269,11 +307,12 @@ parole_media_list_add (ParoleMediaList *list, ParoleFile *file, gboolean emit, g
  * parole_media_list_files_open:
  * @ParoleMediaList: a #ParoleMediaList
  * @files: a #GSList contains a list of #ParoleFile
+ * @disc: TRUE if files are opened to the disc playlist.
  * @emit: TRUE to emit a play signal.
  * 
  **/
 static void
-parole_media_list_files_open (ParoleMediaList *list, GSList *files, gboolean emit)
+parole_media_list_files_open (ParoleMediaList *list, GSList *files, gboolean disc, gboolean emit)
 {
     ParoleFile *file;
     gboolean replace;
@@ -287,21 +326,58 @@ parole_media_list_files_open (ParoleMediaList *list, GSList *files, gboolean emi
     len = g_slist_length (files);
     TRACE ("Adding files");
     
+    if ( len > 1 )
+        g_signal_emit (G_OBJECT (list), signals [SHOW_PLAYLIST], 0, TRUE);
+    
     if ( len != 0 )
     {
 	if ( replace )
 	    parole_media_list_clear_list (list);
 	file = g_slist_nth_data (files, 0);
-	parole_media_list_add (list, file, emit, TRUE);
+	parole_media_list_add (list, file, disc, emit, TRUE);
     }
     
     for ( i = 1; i < len; i++)
     {
 	file = g_slist_nth_data (files, i);
-	parole_media_list_add (list, file, FALSE, FALSE);
+	parole_media_list_add (list, file, disc, FALSE, FALSE);
     }
 }
 
+void
+parole_media_list_add_cdda_tracks (ParoleMediaList *list, gint n_tracks)
+{
+    GSList *files = NULL;
+    ParoleFile *file;
+    int i;
+    
+    for (i = 0; i < n_tracks; i++)
+    {
+        file = parole_file_new_cdda_track( i+1, g_strdup_printf( _("Track %i"), i+1) );
+        files = g_slist_append(files, file);
+    }
+    
+    parole_media_list_files_open(list, files, FALSE, TRUE);
+}
+
+void
+parole_media_list_add_dvd_chapters (ParoleMediaList *list, gint n_chapters)
+{
+    GSList *files = NULL;
+    ParoleFile *file;
+    gint i;
+    
+    for (i = 0; i < n_chapters; i++)
+    {
+        file = PAROLE_FILE(parole_file_new_dvd_chapter( i+1, g_strdup_printf( _("Chapter %i"), i+1) ));
+        files = g_slist_append(files, file);
+    }
+    
+    //parole_media_list_clear_list (list);
+    parole_media_list_files_open(list, files, TRUE, TRUE);
+}
+
+/* Callback to determine whether opened files should start playing immediately */
 static void
 parole_media_list_files_opened_cb (ParoleMediaChooser *chooser, 
 				   GSList *files, 
@@ -313,7 +389,7 @@ parole_media_list_files_opened_cb (ParoleMediaChooser *chooser,
 		  "play-opened-files", &play,
 		  NULL);
     
-    parole_media_list_files_open (list, files, play);
+    parole_media_list_files_open (list, files, FALSE, play);
 }
 
 static void
@@ -328,7 +404,7 @@ parole_media_list_location_opened_cb (ParoleOpenLocation *obj, const gchar *loca
     else
     {
 	file = parole_file_new (location);
-	parole_media_list_add (list, file, TRUE, TRUE);
+	parole_media_list_add (list, file, FALSE, TRUE, TRUE);
     }
 }
 
@@ -388,6 +464,7 @@ parole_media_list_get_files (ParoleMediaList *list)
     return files_list;
 }
 
+/* Callback when an item is dragged on the playlist-widget */
 void	parole_media_list_drag_data_received_cb (GtkWidget *widget,
 						 GdkDragContext *drag_context,
 						 gint x,
@@ -435,18 +512,21 @@ gboolean parole_media_list_key_press (GtkWidget *widget, GdkEventKey *ev, Parole
     return FALSE;
 }
 
+/* Callback for the add button */
 void
 parole_media_list_add_clicked_cb (GtkButton *button, ParoleMediaList *list)
 {
     parole_media_list_open_internal (list);
 }
 
+/* Callback for the clear button */
 void 
 parole_media_list_clear_clicked_cb (GtkButton *button, ParoleMediaList *list)
 {
 	parole_media_list_clear_list (list);
 }
 
+/* Callback for the repeat togglebutton */
 void
 parole_media_list_repeat_toggled_cb (GtkToggleButton *button, ParoleMediaList *list)
 {
@@ -457,6 +537,7 @@ parole_media_list_repeat_toggled_cb (GtkToggleButton *button, ParoleMediaList *l
     g_signal_emit (G_OBJECT (list), signals [REPEAT_TOGGLED], 0, toggled);
 }
 
+/* Callback for the shuffle togglebutton */
 void
 parole_media_list_shuffle_toggled_cb (GtkToggleButton *button, ParoleMediaList *list)
 {
@@ -510,6 +591,34 @@ parole_media_list_get_first_selected_row (ParoleMediaList *list)
     return row;
 }
 
+/**
+ * parole_media_list_get_first_selected_file:
+ * @list: a #ParoleMediaList
+ *
+ * Get the first selected #ParoleFile media file in the media list view
+ *
+ * Returns: a #ParoleFile
+ **/
+static ParoleFile *
+parole_media_list_get_first_selected_file (ParoleMediaList *list)
+{
+    ParoleFile *file;
+    GtkTreeRowReference *row;
+    GtkTreeIter iter;
+
+    row = parole_media_list_get_first_selected_row(list);
+    
+    if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), 
+                                  &iter, 
+                                  gtk_tree_row_reference_get_path (row)) )
+    {
+	    gtk_tree_model_get (GTK_TREE_MODEL (list->priv->store), &iter, DATA_COL, &file, -1);
+    }
+    
+    return file;
+}
+
+/* Callback to save the current playlist */
 void parole_media_list_save_playlist_cb (GtkButton *button, ParolePlaylistSave *data)
 {
     ParolePlFormat format = PAROLE_PL_FORMAT_UNKNOWN;
@@ -561,7 +670,7 @@ out:
     g_free (dirname);
 }
 
-
+/* Query to get the data to populate the tooltip */
 gboolean	parole_media_list_query_tooltip		(GtkWidget *widget,
 							 gint x,
 							 gint y,
@@ -649,6 +758,7 @@ void parole_media_list_format_cursor_changed_cb (GtkTreeView *view, ParolePlayli
     
 }
 
+/* Callback to save the current playlist */
 void parole_media_list_save_cb (GtkButton *button, ParoleMediaList *list)
 {
     ParolePlaylistSave *data;
@@ -775,11 +885,7 @@ parole_media_list_paths_to_row_list (GList *path_list, GtkTreeModel *model)
     return row_list;
 }
 
-/**
- * parole_media_list_remove_clicked_cb:
- * 
- * 
- **/
+/* Callback for the remove-from-playlist button */
 void
 parole_media_list_remove_clicked_cb (GtkButton *button, ParoleMediaList *list)
 {
@@ -816,7 +922,7 @@ parole_media_list_remove_clicked_cb (GtkButton *button, ParoleMediaList *list)
 	
 	if ( gtk_tree_path_prev (prev) )
 	{
-	    parole_media_list_select_path (list, prev);
+	    parole_media_list_select_path (list, FALSE, prev);
 	    row_selected = TRUE;
 	}
 	gtk_tree_path_free (prev);
@@ -849,7 +955,7 @@ parole_media_list_remove_clicked_cb (GtkButton *button, ParoleMediaList *list)
     {
 	GtkTreePath *path;
 	path = parole_media_list_get_first_path (model);
-	parole_media_list_select_path (list, path);
+	parole_media_list_select_path (list, FALSE, path);
 	gtk_tree_path_free (path);
     }
     
@@ -882,8 +988,12 @@ parole_media_list_row_activated_cb (GtkTreeView *view, GtkTreePath *path,
 {
     GtkTreeRowReference *row;
     
+    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
     row = gtk_tree_row_reference_new (gtk_tree_view_get_model (GTK_TREE_VIEW (list->priv->view)), 
 				      path);
+    else
+    row = gtk_tree_row_reference_new (gtk_tree_view_get_model (GTK_TREE_VIEW (list->priv->disc_view)), 
+				      path);
 				      
     g_signal_emit (G_OBJECT (list), signals [MEDIA_ACTIVATED], 0, row);
 }
@@ -978,12 +1088,26 @@ parole_media_list_add_open_containing_folder (ParoleMediaList *list, GtkWidget *
     }
 }
 
+void
+parole_media_list_set_playlist_view(ParoleMediaList *list, gint view)
+{
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook), view);
+    gtk_widget_set_sensitive(list->priv->playlist_controls, !view);
+}
 
-static void
+void
+parole_media_list_clear_disc_list (ParoleMediaList *list)
+{
+    gtk_list_store_clear (GTK_LIST_STORE (list->priv->disc_store));
+}
+
+void
 parole_media_list_clear_list (ParoleMediaList *list)
 {
+    TRACE("CLEAR START");
     gtk_list_store_clear (GTK_LIST_STORE (list->priv->store));
     parole_media_list_set_widget_sensitive (list, FALSE);
+    TRACE("CLEAR END");
 }
 
 static void
@@ -1039,8 +1163,11 @@ parole_media_list_menu_pos (GtkMenu *menu, gint *px, gint *py, gpointer data)
 	window = gtk_widget_get_window( widget );
 	gdk_window_get_position( window, &x, &y );
 	
-	px += x + widget_allocation.x - menu_allocation.width + widget_allocation.width;
-	py += y + widget_allocation.y + widget_allocation.height;
+	if (widget_allocation.width > 100)
+	gtk_widget_set_size_request(GTK_WIDGET(menu), widget_allocation.width, -1);
+	
+	*px = x + widget_allocation.x;
+	*py = y + widget_allocation.y + widget_allocation.height;
 }
 
 static void
@@ -1070,6 +1197,100 @@ menu_detach( GtkMenu *menu )
 	//gtk_menu_detach (menu);
 }
 
+void
+parole_media_list_add_dvd (ParoleMediaList *list, gchar *dvd_name)
+{
+    //parole_media_list_set_dvd_menu_visible(list, TRUE);
+    
+    gtk_label_set_label(GTK_LABEL(list->priv->dvd_label), dvd_name);
+}
+
+void
+parole_media_list_send_dvd_menu_navigation (ParoleMediaList *list, gint msg_id)
+{
+    g_signal_emit (G_OBJECT (list), signals [GST_DVD_NAV_MESSAGE], 0, msg_id);
+}
+
+void
+parole_media_list_dvd_menu_activated (GtkMenuItem *widget, ParoleMediaList *list)
+{
+    parole_media_list_send_dvd_menu_navigation(list, GST_DVD_ROOT_MENU);
+}
+
+void
+parole_media_list_dvd_title_activated (GtkMenuItem *widget, ParoleMediaList *list)
+{
+    parole_media_list_send_dvd_menu_navigation(list, GST_DVD_TITLE_MENU);
+}
+
+void
+parole_media_list_dvd_audio_activated (GtkMenuItem *widget, ParoleMediaList *list)
+{
+    parole_media_list_send_dvd_menu_navigation(list, GST_DVD_AUDIO_MENU);
+}
+
+void
+parole_media_list_dvd_angle_activated (GtkMenuItem *widget, ParoleMediaList *list)
+{
+    parole_media_list_send_dvd_menu_navigation(list, GST_DVD_ANGLE_MENU);
+}
+
+void
+parole_media_list_dvd_chapter_activated (GtkMenuItem *widget, ParoleMediaList *list)
+{
+    parole_media_list_send_dvd_menu_navigation(list, GST_DVD_CHAPTER_MENU);
+}
+
+
+static void
+parole_media_list_show_dvd_menu (GtkToggleButton *button, ParoleMediaList *list)
+{
+    gboolean toggled = gtk_toggle_button_get_active( button );
+    GtkBuilder *builder;
+	GtkMenu *menu;
+	GtkMenuItem *dvd_menu, *title_menu, *audio_menu, *angle_menu, *chapter_menu;
+	
+	if (!toggled)
+	return;
+	
+	builder = parole_builder_new_from_string (playlist_ui, playlist_ui_length);
+    
+    menu = GTK_MENU (gtk_builder_get_object (builder, "dvd-menu"));
+    
+    dvd_menu =   GTK_MENU_ITEM (gtk_builder_get_object (builder, "dvd-menu-menu"));
+    g_signal_connect (dvd_menu, "activate",
+                      G_CALLBACK (parole_media_list_dvd_menu_activated), list);
+                      
+    title_menu = GTK_MENU_ITEM (gtk_builder_get_object (builder, "dvd-menu-title"));
+    g_signal_connect (title_menu, "activate",
+                      G_CALLBACK (parole_media_list_dvd_title_activated), list);
+                      
+    audio_menu = GTK_MENU_ITEM (gtk_builder_get_object (builder, "dvd-menu-audio"));
+    g_signal_connect (audio_menu, "activate",
+                      G_CALLBACK (parole_media_list_dvd_audio_activated), list);
+                      
+    angle_menu = GTK_MENU_ITEM (gtk_builder_get_object (builder, "dvd-menu-angle"));
+    g_signal_connect (angle_menu, "activate",
+                      G_CALLBACK (parole_media_list_dvd_angle_activated), list);
+                      
+    chapter_menu = GTK_MENU_ITEM (gtk_builder_get_object (builder, "dvd-menu-chapter"));
+    g_signal_connect (chapter_menu, "activate",
+                      G_CALLBACK (parole_media_list_dvd_chapter_activated), list);
+    
+    gtk_menu_attach_to_widget( GTK_MENU(menu), list->priv->dvd_menu_button, (GtkMenuDetachFunc) menu_detach );
+    
+    g_signal_connect_swapped (menu, "selection-done",
+                              G_CALLBACK (parole_media_list_destroy_menu), menu);
+                              
+	g_signal_connect_swapped (menu, "destroy",
+                              G_CALLBACK (parole_media_list_hide_menu), list->priv->dvd_menu_button);
+    
+    gtk_menu_popup (GTK_MENU (menu), 
+                    NULL, NULL,
+                    (GtkMenuPositionFunc) parole_media_list_menu_pos, NULL,
+                    3, gtk_get_current_event_time ());
+}
+
 static void
 parole_media_list_show_button_menu (GtkToggleButton *button, ParoleMediaList *list)
 {
@@ -1236,10 +1457,18 @@ parole_media_list_button_release_event (GtkWidget *widget, GdkEventButton *ev, P
 }
 
 static void
-parole_media_list_select_path (ParoleMediaList *list, GtkTreePath *path)
+parole_media_list_select_path (ParoleMediaList *list, gboolean disc, GtkTreePath *path)
 {
+    if (disc)
+    {
+    gtk_tree_selection_select_path (list->priv->disc_sel, path);
+    gtk_tree_view_set_cursor (GTK_TREE_VIEW (list->priv->disc_view), path, NULL, FALSE);
+    }
+    else
+    {
     gtk_tree_selection_select_path (list->priv->sel, path);
     gtk_tree_view_set_cursor (GTK_TREE_VIEW (list->priv->view), path, NULL, FALSE);
+    }
 }
 
 static GtkTreeRowReference *
@@ -1252,7 +1481,7 @@ parole_media_list_get_row_reference_from_iter (ParoleMediaList *list, GtkTreeIte
     row = gtk_tree_row_reference_new (GTK_TREE_MODEL (list->priv->store), path);
     
     if ( select_path)
-	parole_media_list_select_path (list, path);
+	parole_media_list_select_path (list, FALSE, path);
     
     gtk_tree_path_free (path);
     
@@ -1322,6 +1551,24 @@ parole_media_list_class_init (ParoleMediaListClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__BOOLEAN,
                       G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+                      
+    signals[SHOW_PLAYLIST] = 
+        g_signal_new ("show-playlist",
+                      PAROLE_TYPE_MEDIA_LIST,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ParoleMediaListClass, show_playlist),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__BOOLEAN,
+                      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+                      
+    signals[GST_DVD_NAV_MESSAGE] = 
+        g_signal_new ("gst-dvd-nav-message",
+                      PAROLE_TYPE_MEDIA_LIST,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (ParoleMediaListClass, gst_dvd_nav_message),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__INT,
+                      G_TYPE_NONE, 1, G_TYPE_INT);
 
     g_type_class_add_private (klass, sizeof (ParoleMediaListPrivate));
     
@@ -1331,60 +1578,80 @@ parole_media_list_class_init (ParoleMediaListClass *klass)
 static void
 parole_media_list_setup_view (ParoleMediaList *list)
 {
-    GtkTreeSelection *sel;
-    GtkListStore *list_store;
-    GtkTreeViewColumn *col;
-    GtkCellRenderer *renderer;
+    GtkTreeSelection *sel, *disc_sel;
+    GtkListStore *list_store, *disc_list_store;
+    GtkTreeViewColumn *col, *disc_col;
+    GtkCellRenderer *renderer, *disc_renderer;
 
     list_store = gtk_list_store_new (COL_NUMBERS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT);
+    disc_list_store = gtk_list_store_new (COL_NUMBERS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT);
 
     gtk_tree_view_set_model (GTK_TREE_VIEW (list->priv->view), GTK_TREE_MODEL(list_store));
+    gtk_tree_view_set_model (GTK_TREE_VIEW (list->priv->disc_view), GTK_TREE_MODEL(disc_list_store));
     
     gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list->priv->view), TRUE);
+    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (list->priv->disc_view), TRUE);
     col = gtk_tree_view_column_new ();
+    disc_col = gtk_tree_view_column_new ();
 
     renderer = gtk_cell_renderer_pixbuf_new ();
+    disc_renderer = gtk_cell_renderer_pixbuf_new ();
     
     gtk_tree_view_column_pack_start(col, renderer, FALSE);
+    gtk_tree_view_column_pack_start(disc_col, disc_renderer, FALSE);
     gtk_tree_view_column_set_attributes(col, renderer, "pixbuf", PIXBUF_COL, NULL);
-
+    gtk_tree_view_column_set_attributes(disc_col, disc_renderer, "pixbuf", PIXBUF_COL, NULL);
 
     /**
      * Name col
      * 
      **/
     renderer = gtk_cell_renderer_text_new();
+    disc_renderer = gtk_cell_renderer_text_new();
     
     gtk_tree_view_column_pack_start (col, renderer, TRUE);
     gtk_tree_view_column_set_attributes (col, renderer, "text", NAME_COL, NULL);
     g_object_set (renderer, 
 		  "ellipsize", PANGO_ELLIPSIZE_END, 
 		  NULL);
-    
+		  
+    gtk_tree_view_column_pack_start (disc_col, disc_renderer, TRUE);
+    gtk_tree_view_column_set_attributes (disc_col, disc_renderer, "text", NAME_COL, NULL);
+    g_object_set (disc_renderer, 
+		  "ellipsize", PANGO_ELLIPSIZE_END, 
+		  NULL);
     
     /**
      * Media length
      * 
      **/
     renderer = gtk_cell_renderer_text_new();
+    disc_renderer = gtk_cell_renderer_text_new();
     
     gtk_tree_view_column_pack_start (col, renderer, FALSE);
+    gtk_tree_view_column_pack_start (disc_col, disc_renderer, FALSE);
     gtk_tree_view_column_set_attributes (col, renderer, "text", LENGTH_COL, NULL);
-    
+    gtk_tree_view_column_set_attributes (disc_col, disc_renderer, "text", LENGTH_COL, NULL);
     
     gtk_tree_view_append_column (GTK_TREE_VIEW (list->priv->view), col);
+    gtk_tree_view_append_column (GTK_TREE_VIEW (list->priv->disc_view), disc_col);
     gtk_tree_view_column_set_title (col, _("Media list"));
+    gtk_tree_view_column_set_title (disc_col, _("Chapter list"));
 
     gtk_drag_dest_set (list->priv->view, GTK_DEST_DEFAULT_ALL, target_entry, G_N_ELEMENTS (target_entry),
                        GDK_ACTION_COPY | GDK_ACTION_MOVE);
     
     list->priv->sel = sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list->priv->view));
+    list->priv->disc_sel = disc_sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list->priv->disc_view));
     gtk_tree_selection_set_mode (sel, GTK_SELECTION_MULTIPLE);
     
     g_signal_connect (sel, "changed",
 		      G_CALLBACK (parole_media_list_selection_changed_cb), list);
+    g_signal_connect (disc_sel, "changed",
+		      G_CALLBACK (parole_media_list_selection_changed_cb), list);
     
     list->priv->store = list_store;
+    list->priv->disc_store = disc_list_store;
 }
 
 static void
@@ -1401,7 +1668,17 @@ parole_media_list_init (ParoleMediaList *list)
     
     builder = parole_builder_new_from_string (playlist_ui, playlist_ui_length);
     
+    list->priv->dvd_menu_button = GTK_WIDGET (gtk_builder_get_object(builder, "dvd_menu_button"));
+    list->priv->dvd_menu = GTK_WIDGET (gtk_builder_get_object(builder, "dvd-menu"));
+    list->priv->dvd_label = GTK_WIDGET (gtk_builder_get_object(builder, "dvd_label"));
+    list->priv->playlist_controls = GTK_WIDGET (gtk_builder_get_object(builder, "playlist_controls"));
+    list->priv->playlist_notebook = GTK_WIDGET (gtk_builder_get_object(builder, "playlist_notebook"));
+    
+    g_signal_connect (GTK_TOGGLE_BUTTON(list->priv->dvd_menu_button), "toggled",
+		      G_CALLBACK (parole_media_list_show_dvd_menu), list);
+    
     list->priv->view = GTK_WIDGET (gtk_builder_get_object (builder, "media-list"));
+    list->priv->disc_view = GTK_WIDGET (gtk_builder_get_object (builder, "disc-list"));
     
     box = GTK_WIDGET (gtk_builder_get_object (builder, "playlist-box"));
     
@@ -1453,10 +1730,9 @@ void parole_media_list_load (ParoleMediaList *list)
     
     g_object_get (G_OBJECT (list->priv->conf),
 		  "play-opened-files", &play,
+		  "remember-playlist", &load_saved_list,
 		  NULL);
     
-    load_saved_list = parole_rc_read_entry_bool ("SAVE_LIST_ON_EXIT", PAROLE_RC_GROUP_GENERAL, FALSE);
-    
     if ( load_saved_list )
     {
 	gchar *playlist_file;
@@ -1469,7 +1745,7 @@ void parole_media_list_load (ParoleMediaList *list)
 	    fileslist = parole_pl_parser_parse_from_file_by_extension (playlist_file);
 	    g_free (playlist_file);
 	    
-	    parole_media_list_files_open (list, fileslist, play);
+	    parole_media_list_files_open (list, fileslist, FALSE, play);
 	    g_slist_free (fileslist);
 	}
     }
@@ -1491,7 +1767,7 @@ parole_media_list_add_by_path (ParoleMediaList *list, const gchar *path, gboolea
     
     parole_get_media_files (filter, path, TRUE, &files_list);
     
-    parole_media_list_files_open (list, files_list, emit);
+    parole_media_list_files_open (list, files_list, FALSE, emit);
     
     len = g_slist_length (files_list);
     ret = len == 0 ? FALSE : TRUE;
@@ -1565,6 +1841,37 @@ GtkTreeRowReference *parole_media_list_get_prev_row (ParoleMediaList *list,
     return prev;
 }
 
+GtkTreeRowReference *parole_media_list_get_row_n (ParoleMediaList *list, 
+						                          gint wanted_row)
+{
+    GtkTreeRowReference *row = NULL;
+    GtkTreePath *path;
+    GtkTreeIter iter;
+    
+    if (wanted_row == -1)
+        return NULL;
+        
+    path = gtk_tree_path_new_from_string( g_strdup_printf("%i", wanted_row) );
+
+    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
+    {
+        if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path))
+        	row = gtk_tree_row_reference_new (GTK_TREE_MODEL (list->priv->store), path);
+    }
+    else
+    {
+        if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->disc_store), &iter, path))
+        	row = gtk_tree_row_reference_new (GTK_TREE_MODEL (list->priv->disc_store), path);
+    }
+    
+    gtk_tree_path_free (path);
+    
+    if ( !gtk_tree_row_reference_valid (row) )
+	return NULL;
+    
+    return row;
+}
+
 GtkTreeRowReference *parole_media_list_get_row_random (ParoleMediaList *list)
 {
     GtkTreeRowReference *row = NULL;
@@ -1597,15 +1904,21 @@ GtkTreeRowReference *parole_media_list_get_row_random (ParoleMediaList *list)
 }
 
 gboolean parole_media_list_is_selected_row  (ParoleMediaList *list)
-{
+{   
+    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
     return gtk_tree_selection_count_selected_rows (list->priv->sel) > 0;
+    else
+    return gtk_tree_selection_count_selected_rows (list->priv->disc_sel) > 0;
 }
 
 gboolean parole_media_list_is_empty (ParoleMediaList *list)
 {
     GtkTreeIter iter;
     
+    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
     return !gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->priv->store), &iter);
+    else
+    return !gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->priv->disc_store), &iter);
 }
 
 /**
@@ -1620,9 +1933,15 @@ GtkTreeRowReference *parole_media_list_get_first_row (ParoleMediaList *list)
     GtkTreeRowReference *row = NULL;
     GtkTreeIter iter;
     
-    if ( gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->priv->store), &iter) )
+    if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
     {
-	row = parole_media_list_get_row_reference_from_iter (list, &iter, TRUE);
+        if ( gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->priv->store), &iter) )
+	        row = parole_media_list_get_row_reference_from_iter (list, &iter, TRUE);
+    }
+    else
+    {
+        if ( gtk_tree_model_get_iter_first (GTK_TREE_MODEL (list->priv->disc_store), &iter) )
+    	    row = parole_media_list_get_row_reference_from_iter (list, &iter, TRUE);
     }
     
     return row;
@@ -1640,15 +1959,29 @@ GtkTreeRowReference *parole_media_list_get_selected_row (ParoleMediaList *list)
     return parole_media_list_get_first_selected_row (list);
 }
 
+/**
+ * parole_media_list_get_selected_file:
+ * @list: a #ParoleMediaList
+ * 
+ * 
+ * Returns: a #ParoleFile of the selected row.
+ **/
+ParoleFile *parole_media_list_get_selected_file (ParoleMediaList *list)
+{
+    return parole_media_list_get_first_selected_file (list);
+}
+
 void parole_media_list_select_row (ParoleMediaList *list, GtkTreeRowReference *row)
 {
     GtkTreePath *path;
     
     if ( gtk_tree_row_reference_valid (row) )
     {
-	path = gtk_tree_row_reference_get_path (row);
-	parole_media_list_select_path (list, path);
-	gtk_tree_path_free (path);
+	    path = gtk_tree_row_reference_get_path (row);
+	    parole_media_list_select_path (list, 
+	        gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 1, 
+	        path);
+	    gtk_tree_path_free (path);
     }
 }
 
@@ -1661,10 +1994,17 @@ void parole_media_list_set_row_pixbuf  (ParoleMediaList *list, GtkTreeRowReferen
     {
 	path = gtk_tree_row_reference_get_path (row);
 	
-	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+	if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
 	{
-	    gtk_list_store_set (list->priv->store, &iter, PIXBUF_COL, pix, -1);
+	    if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+	        gtk_list_store_set (list->priv->store, &iter, PIXBUF_COL, pix, -1);
 	}
+	else
+	{
+	    if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->disc_store), &iter, path) )
+	        gtk_list_store_set (list->priv->disc_store, &iter, PIXBUF_COL, pix, -1);
+	}
+
 	gtk_tree_path_free (path);
     }
 }
@@ -1678,10 +2018,17 @@ void parole_media_list_set_row_name (ParoleMediaList *list, GtkTreeRowReference
     {
 	path = gtk_tree_row_reference_get_path (row);
 	
-	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+	if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
 	{
-	    gtk_list_store_set (list->priv->store, &iter, NAME_COL, name, -1);
+    	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+    	    gtk_list_store_set (list->priv->store, &iter, NAME_COL, name, -1);
 	}
+	else
+	{
+    	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->disc_store), &iter, path) )
+    	    gtk_list_store_set (list->priv->disc_store, &iter, NAME_COL, name, -1);
+	}
+	
 	gtk_tree_path_free (path);
     }
 }
@@ -1695,10 +2042,17 @@ void parole_media_list_set_row_length (ParoleMediaList *list, GtkTreeRowReferenc
     {
 	path = gtk_tree_row_reference_get_path (row);
 	
-	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+	if (gtk_notebook_get_current_page(GTK_NOTEBOOK(list->priv->playlist_notebook)) == 0)
 	{
-	    gtk_list_store_set (list->priv->store, &iter, LENGTH_COL, len, -1);
+	    if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->store), &iter, path) )
+    	    gtk_list_store_set (list->priv->store, &iter, LENGTH_COL, len, -1);
 	}
+	else
+	{
+    	if ( gtk_tree_model_get_iter (GTK_TREE_MODEL (list->priv->disc_store), &iter, path) )
+    	    gtk_list_store_set (list->priv->disc_store, &iter, LENGTH_COL, len, -1);
+	}
+	
 	gtk_tree_path_free (path);
     }
 }
@@ -1733,9 +2087,9 @@ gboolean parole_media_list_add_files (ParoleMediaList *list, gchar **filenames,
 	    TRACE ("File=%s", filenames[i]);
 	    file = parole_file_new (filenames[i]);
 		if (enqueue) {
-			parole_media_list_add (list, file, FALSE, FALSE);}
+			parole_media_list_add (list, file, FALSE, FALSE, FALSE);}
 		else
-			parole_media_list_add (list, file, i == 0 ? TRUE : FALSE, i == 0 ? TRUE : FALSE);
+			parole_media_list_add (list, file, FALSE, i == 0 ? TRUE : FALSE, i == 0 ? TRUE : FALSE);
 	    added++;
 	}
     }
@@ -1747,7 +2101,9 @@ void parole_media_list_save_list (ParoleMediaList *list)
 {
     gboolean save;
     
-    save = parole_rc_read_entry_bool ("SAVE_LIST_ON_EXIT", PAROLE_RC_GROUP_GENERAL, FALSE);
+    g_object_get (G_OBJECT (list->priv->conf),
+		  "remember-playlist", &save,
+		  NULL);
     
     if ( save )
     {
