@@ -22,6 +22,7 @@
 #define __PAROLE_MEDIA_LIST_H
 
 #include <gtk/gtk.h>
+#include <src/misc/parole-file.h>
 
 G_BEGIN_DECLS
 
@@ -38,6 +39,12 @@ enum
     COL_NUMBERS
 };
 
+enum
+{
+    PAROLE_MEDIA_LIST_PLAYLIST_VIEW_STANDARD,
+    PAROLE_MEDIA_LIST_PLAYLIST_VIEW_DISC
+};
+
 typedef struct ParoleMediaListPrivate ParoleMediaListPrivate;
 
 typedef struct
@@ -66,6 +73,12 @@ typedef struct
 								     
 	void			(*repeat_toggled)		    (ParoleMediaList *list,
 								     gboolean repeat_toggled);
+								     
+    void			(*show_playlist)		    (ParoleMediaList *list,
+								     gboolean show_playlist);
+								     
+    void			(*gst_dvd_nav_message)		    (ParoleMediaList *list,
+								     gint gst_dvd_nav_message);
     
 } ParoleMediaListClass;
 
@@ -75,6 +88,12 @@ GtkWidget       		*parole_media_list_get              (void);
 
 void				 parole_media_list_load             (ParoleMediaList *list);
 
+void    parole_media_list_set_playlist_view(ParoleMediaList *list, gint view);
+
+void    parole_media_list_clear_disc_list (ParoleMediaList *list);
+
+void	parole_media_list_clear_list 	  (ParoleMediaList *list);
+
 
 gboolean			 parole_media_list_add_by_path      (ParoleMediaList *list, 
 								     const gchar *path, 
@@ -88,6 +107,8 @@ GtkTreeRowReference             *parole_media_list_get_first_row    (ParoleMedia
 
 GtkTreeRowReference		*parole_media_list_get_selected_row (ParoleMediaList *list);
 
+ParoleFile              *parole_media_list_get_selected_file (ParoleMediaList *list);
+
 void				 parole_media_list_select_row 	    (ParoleMediaList *list,
 								     GtkTreeRowReference *row);
 
@@ -119,6 +140,13 @@ void			         parole_media_list_open_location    (ParoleMediaList *list);
 
 gboolean			 parole_media_list_add_files        (ParoleMediaList *list,
 								     gchar **filenames, gboolean enqueue);
+								     
+void                parole_media_list_add_cdda_tracks (ParoleMediaList *list, gint n_tracks);
+
+void                parole_media_list_add_dvd_chapters (ParoleMediaList *list, gint n_chapters);
+
+GtkTreeRowReference *parole_media_list_get_row_n (ParoleMediaList *list, 
+						                          gint wanted_row);
 
 void				 parole_media_list_save_list	    (ParoleMediaList *list);
 
@@ -129,6 +157,9 @@ void				 parole_media_list_set_repeat_toggled		(ParoleMediaList *list,
 																
 void 				 parole_media_list_set_shuffle_toggled		(ParoleMediaList *list,
 																gboolean shuffle_toggled);
+																
+void parole_media_list_add_dvd (ParoleMediaList *list, gchar *dvd_name);
+
 
 G_END_DECLS
 
